from __future__ import print_function
import os
import re
import sys


def trim_adaptors(fastq_dir, strain_barcode_dict, barcode_kits='EXP-NBD104', outdir='', thread=16, demultiplexing=False):
    """
    Trim adaptors and demultiplexing using guppy_barcoder and qcat
    """
    try:
        with open(os.path.join(outdir, '1.demultiplexing_trimming.sh'), 'w') as f:
            flag = 0
            if os.path.isdir(fastq_dir):
                for item in os.listdir(fastq_dir):
                    if item.endswith('.fastq.gz') or item.endswith('.fastq') or item.endswith('.fq.gz') or item.endswith('.fq'):
                        flag = 1
                        break
                if flag == 0:
                    print('Error: Not found ".fastq(.gz)" or ".fq(.gz)" format files in dir {}.'.format(fastq_dir))
                    sys.exit(-1)
        
            if demultiplexing is False:
                for key in strain_barcode_dict.keys():
                    tmp_path = os.path.join(outdir, 'tmp.fastq')
                    prefix = os.path.join(outdir, 'trimmed_{}'.format(strain_barcode_dict[key][1]))
                    f.write('cat {} >{} | qcat --trim --detect-middle -f {} -o {}.fastq\nrm {}\n'.format(os.path.join(fastq_dir, '*.fastq'), tmp_path, tmp_path, prefix, tmp_path))
                # f.write('cat {} >{} | qcat --trim --detect-middle -f {} -o {}/trimmed_sample.fastq\nrm {}\n'.format(os.path.join(fastq_dir, '*.fastq'), os.path.join(outdir, 'tmp.fastq'), outdir, os.path.join(outdir, 'tmp.fastq'), os.path.join(outdir, 'tmp.fastq')))

            else:
                f.write('guppy_barcoder -i {} --barcode_kits {} -s {}/barcode_demultiplexing_data --min_score 80 -q 100000000 -t {}\n'.format(fastq_dir, barcode_kits, outdir, thread))

                if barcode_kits == 'EXP-NBD103' or barcode_kits == 'EXP-NBD104':
                    barcode_kits_qcat = 'NBD103/NBD104'
                elif barcode_kits == 'EXP-NBD114':
                    barcode_kits_qcat = 'NBD114'
                else:
                    barcode_kits_qcat = 'NBD104/NBD114'

                for key in strain_barcode_dict.keys():
                    print('qcat -k {} --trim --detect-middle -f {}/barcode_demultiplexing_data/{}/*.fastq -o {}/barcode_demultiplexing_data/trimmed_{}.fastq\n'.format(barcode_kits_qcat, outdir, key, outdir, strain_barcode_dict[key][1]))
                    f.write('qcat -k {} --trim --detect-middle -f {}/barcode_demultiplexing_data/{}/*.fastq -o {}/barcode_demultiplexing_data/trimmed_{}.fastq\n'.format(barcode_kits_qcat, outdir, key, outdir, strain_barcode_dict[key][1]))

        print('####### Demultiplexing Using Guppy and Trimming Adaptors Using Qcat ########')
        os.system('sh {}/1.demultiplexing_trimming.sh'.format(outdir))
    except Exception as e:
        raise e


def quality_control_pauvre(outdir='', prefix=None):
    """
    Quality control using pauvre
    """
    try:
        with open(os.path.join(outdir, '3.quality_control_pauvre.sh'), 'w') as f:
            if os.path.isdir('{}/barcode_demultiplexing_data'.format(outdir)):
                if not os.path.exists(os.path.join(outdir, 'quality_control')):
                    os.makedirs(os.path.join(outdir, 'quality_control'))
                for item in os.listdir('{}/barcode_demultiplexing_data'.format(outdir)):
                    if item.endswith('.fastq.gz') or item.endswith('.fastq') or item.endswith('.fq.gz') or item.endswith('.fq'):
                        prefix = re.sub('.fastq.gz|.fastq|.fq.gz|.fq|.gz', '', os.path.basename(item))
                        tmp_name = '{} read length vs mean quality'.format(prefix)
                        f.write('pauvre marginplot -f {}/barcode_demultiplexing_data/{} -y -t \"{}\" -o {}_QCstat >{}/quality_control/{}_QCstat.out && mv *.png {}/quality_control\n'.format(outdir, item, tmp_name, prefix, outdir, prefix, outdir))
            else:
               for item in os.listdir(outdir):
                    if item.startswith('trimmed_') and (item.endswith('.fastq.gz') or item.endswith('.fastq') or item.endswith('.fq.gz') or item.endswith('.fq')):
                        prefix = re.sub('.fastq.gz|.fastq|.fq.gz|.fq|.gz', '', os.path.basename(item))
                        tmp_name = '{} read length vs mean quality'.format(prefix)
                        f.write('pauvre marginplot -f {}/{} -y -t \"{}\" -o QCstat >{}/QCstat.out && mv *.png {}\n'.format(outdir, item, tmp_name, outdir, outdir))                
                    elif item.startswith('filtered_') and (item.endswith('.fastq.gz') or item.endswith('.fastq') or item.endswith('.fq.gz') or item.endswith('.fq')):
                        prefix = re.sub('.fastq.gz|.fastq|.fq.gz|.fq|.gz', '', os.path.basename(item))
                        tmp_name = '{} read length vs mean quality'.format(prefix)
                        f.write('pauvre marginplot -f {}/{} -y -t \"{}\" -o filtered_QCstat >{}/filtered_QCstat.out && mv *.png {}\n'.format(outdir, item, tmp_name, outdir, outdir))                

                # f.write('pauvre marginplot -f {}/trimmed_sample.fastq -y -o QCstat >{}/QCstat.out && mv *.png {}\n'.format(outdir, outdir, outdir))
                # f.write('pauvre marginplot -f {}/filtered_trimmed_sample.fastq -y -o filtered_QCstat >{}/filtered_QCstat.out && mv *.png {}\n'.format(outdir, outdir, outdir))

        print('####### Quality Control Using Pauvre ########')
        os.system('sh {}/3.quality_control_pauvre.sh'.format(outdir))
    except Exception as e:
        raise e


def filter_reads(outdir='', min_quality=7, min_length=500, max_length=10000000):
    try:
        with open(os.path.join(outdir, '2.filter_reads.sh'), 'w') as f:
            if os.path.isdir('{}/barcode_demultiplexing_data'.format(outdir)):
                for item in os.listdir('{}/barcode_demultiplexing_data'.format(outdir)):
                    prefix = os.path.join(outdir, 'barcode_demultiplexing_data', 'filtered_{}'.format(item))
                    if item.endswith('.fastq.gz') or item.endswith('.fq.gz'):
                        f.write('gunzip -c {}/barcode_demultiplexing_data/{} | NanoFilt -q {} -l {} --maxlength {} |gzip > {}\n'.format(outdir, item, min_quality, min_length, max_length, prefix))
                    elif item.endswith('.fastq') or item.endswith('.fq'):
                        f.write('cat {}/barcode_demultiplexing_data/{} | NanoFilt -q {} -l {} --maxlength {} |gzip > {}.gz\n'.format(outdir, item, min_quality, min_length, max_length, prefix))
            else:
                for item in os.listdir(outdir):
                    prefix = os.path.join(outdir, 'filtered_{}'.format(item))
                    if item.endswith('.fastq.gz') or item.endswith('.fq.gz'):
                        f.write('gunzip -c {}/{} | NanoFilt -q {} -l {} --maxlength {} |gzip > {}\n'.format(outdir, item, min_quality, min_length, max_length, prefix))
                    elif item.endswith('.fastq') or item.endswith('.fq'):
                        f.write('cat {}/{} | NanoFilt -q {} -l {} --maxlength {} |gzip > {}.gz\n'.format(outdir, item, min_quality, min_length, max_length, prefix))
                # f.write('cat {}/trimmed_sample.fastq | NanoFilt -q {} -l {} --maxlength {} > {}/filtered_trimmed_sample.fastq\n'.format(outdir, min_quality, min_length, max_length, outdir, outdir))

        print('####### Filter Low Quality and Short Reads Using NanoFilt ########')
        os.system('sh {}/2.filter_reads.sh'.format(outdir))
    except Exception as e:
        raise e


def basic_data_analysis(input, strain_barcode_dict, outdir=None, flowcell='FLO-MIN106', kit='SQK-LSK109', barcode_kits='EXP-NBD104', thread=16, barcoding=False, min_quality=7, min_length=500, max_length=1000000000):
    if outdir is None or outdir == '':
        outdir = '.' 
    else:
        outdir = outdir
        if outdir.endswith('/'):
            outdir = re.sub('/$', '', outdir)
        if not os.path.isdir(outdir):
            os.makedirs(outdir)

    trim_adaptors(fastq_dir=input, strain_barcode_dict=strain_barcode_dict, barcode_kits=barcode_kits, outdir=outdir, thread=thread, demultiplexing=barcoding)

    filter_reads(outdir, min_quality, min_length, max_length)
    quality_control_pauvre(outdir)



from __future__ import print_function
import argparse
import os
import re
import sys
import textwrap

import basic_data_analysis
import report


VFDB_PATH=os.path.join(sys.path[0], "VFDB_setA_nt.v2.fas")

# ---- 2a Nanopore-only Assembly ----

# Step1: Canu assembly
def canu_assembly(fq_file, genome_size, outdir, prefix='canu_assembly', corrected_error_rate=0.144, minReadLength=1000, minOverlapLength=500):
    try:
        shell_path = os.path.join(outdir, '2-1.canu_assembly.sh')
        outdir = os.path.join(outdir, 'canu_assembly')
        if not os.path.isdir(outdir):
            os.makedirs(outdir)

        with open(shell_path, 'w') as f:
            f.write('canu -p {} -d {} genomesize={} correctedErrorRate={} useGrid=false -nanopore-raw {} minReadLength={} minOverlapLength={}\n'.format(prefix, outdir, genome_size, corrected_error_rate, fq_file, minReadLength, minOverlapLength))

        print('\n---- (2) Start Nanopore-only Assembly ----\n\n---- Step2-1: Canu assembly ----')
        os.system('sh {}'.format(shell_path))
    except Exception as e:
        raise e


# Step2b: Polishing (Racon)
def polish_racon(fa_file, fq_file, outdir, thread=16):
    try:
        shell_path = os.path.join(outdir, '2-2.polish_racon.sh')
        old_outdir = outdir
        outdir = os.path.join(outdir, 'racon_result')
        if not os.path.isdir(outdir):
            os.makedirs(outdir)
        with open(shell_path, 'w') as f:
            # bwa
            bam_path = os.path.join(outdir, 'reads.sorted.bam')
            sam_path = re.sub('.bam', '.sam', bam_path)
            tmp_fa_path = os.path.join(outdir, 'racon_tmp.fa')
            final_Fa_path = os.path.join(outdir, 'racon_polished.fa')

            f.write('bwa index {}\n'.format(fa_file))
            f.write('bwa mem -x ont2d -t {} {} {} | samtools sort -o {} -T reads.tmp -\n'.format(thread, fa_file, fq_file, bam_path))
            f.write('samtools view -h {} > {}\n'.format(bam_path, sam_path))

            # racon consensus
            f.write('racon {} {} {} -t {} > {} \n'.format(fq_file, sam_path, fa_file, thread, tmp_fa_path))
            f.write('grep -v "racon" {} | dos2unix > {}\n'.format(tmp_fa_path, final_Fa_path))
            f.write('cp {} {}/assembly.fasta\n'.format(final_Fa_path, old_outdir))
            f.write('rm {}\n'.format(tmp_fa_path))

        print('---- Step2-2: Polishing using Racon ----')
        os.system('sh {}'.format(shell_path))
    except Exception as e:
        raise e

# ---- 2b Hybrid Assembly ----

# Step1: Hybrid assembly (Unicycler)
def unicycler_hybrid_assembly(fq_file, short_r1, short_r2, outdir, mode='normal'):
    try:
        if not os.path.isdir(outdir):
            os.makedirs(outdir)
        shell_path = os.path.join(outdir, '2-1.unicycler_hybrid_assembly.sh')
        with open(shell_path, 'w') as f:
            f.write('unicycler -1 {} -2 {} -l {} --mode {} -o {}\n'.format(short_r1, short_r2, fq_file, mode, outdir))

        print('\n---- (2) Start Nanopore-NGS Hybrid Assembly ----\n\n---- Step2-1: Unicycler Hybrid Assembly ----')
        os.system('sh {}'.format(shell_path))
    except Exception as e:
        raise e


def assembly_one(nanopore_fq_file, strain_info, outdir, thread=16, genomesize=None, minReadLength=1000, minOverlapLength=500):
    for key in strain_info.keys():
        if len(strain_info[key]) == 2:
            if genomesize is None:
                print('Error: parameter "genomesize" is required when using canu software for long-read-only assembly.\n')
                sys.exit(-1)

            canu_assembly_dir = os.path.join(outdir, strain_info[key][1])
            canu_assembly(nanopore_fq_file, genomesize, outdir=canu_assembly_dir, minReadLength=minReadLength, minOverlapLength=minOverlapLength)

            polish_racon(fa_file='{}/*.contigs.fasta'.format(canu_assembly_dir), fq_file=nanopore_fq_file, outdir=outdir, thread=thread)
        elif len(strain_info[key]) == 4:
            if os.path.exists(strain_info[key][2]) and os.path.exists(strain_info[key][3]):
                unicycler_hybrid_assembly(nanopore_fq_file, strain_info[key][2], strain_info[key][3], outdir)
            else:
                print('Error: please check the Illuminia fastq path: {} or {}'.format(strain_info[key][2], strain_info[key][3]))
                sys.exit(-1)


def assembly_all(nanopore_fq_path, strain_info, outdir, thread=16, genomesize=None, minReadLength=1000, minOverlapLength=500):
    for key in strain_info.keys():
        nanopore_fq_file = os.path.join(nanopore_fq_path, 'filtered_trimmed_{}.fastq.gz'.format(strain_info[key][1]))
        if len(strain_info[key]) == 2:
            if genomesize is None:
                print('Error: parameter "genomesize" is required when using canu software for long-read-only assembly.\n')
                sys.exit(-1)

            canu_assembly_dir = os.path.join(outdir, strain_info[key][1])
            canu_assembly(nanopore_fq_file, genomesize, outdir=canu_assembly_dir, minReadLength=minReadLength, minOverlapLength=minOverlapLength)

            polish_racon(fa_file='{}/*.contigs.fasta'.format(os.path.join(canu_assembly_dir, 'canu_assembly')), fq_file=nanopore_fq_file, outdir=canu_assembly_dir, thread=thread)
        elif len(strain_info[key]) == 4:
            if os.path.exists(strain_info[key][2]) and os.path.exists(strain_info[key][3]):
                unicycler_hybrid_assembly(nanopore_fq_file, strain_info[key][2], strain_info[key][3], os.path.join(outdir, strain_info[key][1]))
            else:
                print('Error: please check the Illuminia fastq path: {} or {}'.format(strain_info[key][2], strain_info[key][3]))
                sys.exit(-1)


# ---- 3 Data Analysis after assembly ----

# Step1: Genomic Anotation
def prokka_annotation(fasta_file, prefix, outdir):
    outdir = os.path.join(outdir, '03.Prokka', prefix)
    shell_path = os.path.join(outdir, '3-1.prokka.sh')
    if not os.path.isdir(outdir):
        os.makedirs(outdir)
    with open(shell_path, 'w') as f:
        f.write('prokka --force -outdir {} --prefix {} {}\n'.format(outdir, prefix, fasta_file))
    os.system('sh {}'.format(shell_path))


# Step2: Circlator Analysis and plot

# Step3: AMR gene Identification
def card_annotation(fasta_file, prefix, outdir):
    outdir = os.path.join(outdir, '04.Resistance_genes', prefix)
    shell_path = os.path.join(outdir, '4-1.rgi.sh')
    if not os.path.exists(outdir):
        os.makedirs(outdir)

    with open(shell_path, 'w') as f:
        f.write('rgi main -i {} -o {}/{} --clean'.format(fasta_file, outdir, prefix))
    os.system('sh {}'.format(shell_path))


# Step4: Virulence Factors Identification
def vfdb_annotation(fasta_file, prefix, outdir):
    outdir = os.path.join(outdir, '05.Virulence_genes', prefix)
    shell_path = os.path.join(outdir, '5-1.vfdb.sh')

    if not os.path.isdir(outdir):
        os.makedirs(outdir)

    with open(shell_path, 'w') as f:
        f.write('blastn -query {} -db {} -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore staxids stitle qcovs qcovhsp" -out {}/{}.vfdb.m6.out\n'.format(fasta_file, VFDB_PATH, outdir, prefix))
    os.system('sh {}'.format(shell_path))

    # filter vfdb blast result: identity >= 90%, hit_len/total_gene_len >= 90% and qcovs >= 90% 
    out = open('{}/{}.vfdb.xls'.format(outdir, prefix), 'w')
    with open('{}/{}.vfdb.m6.out'.format(outdir, prefix)) as f:
        out.write('Contig\tStart\tEnd\tVFDB ID\tGene Name\tVirulence Factor\tGene Function\tSpecies\tIdentity\tHit Length\n')
        for line in f.readlines():
            t = line.strip().split('\t')
            s = t[-3].split('|')
            sub_len = int((s[3].split('-'))[-1])
            if (float(t[3]) / sub_len >= 0.9) and (float(t[2]) >= 90):
                s[4] = re.sub('ARO:', '', s[4])
                out.write('{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\n'.format(t[0],t[6],t[7],s[4],s[5],s[6],s[8],s[7],t[2],t[3]))
    out.close()


def annotation(strain_info, outdir):
    for key in strain_info.keys():
        prefix = strain_info[key][1]
        fasta_file = os.path.join(outdir, '02.Assembly', prefix, 'assembly.fasta')

        prokka_annotation(fasta_file, prefix, outdir)
        card_annotation(fasta_file, prefix, outdir)
        vfdb_annotation(fasta_file, prefix, outdir)



# -*- coding: utf-8 -*-

from __future__ import print_function
from collections import defaultdict
import os
import pandas
import sys
import time
import re


reload(sys)
sys.setdefaultencoding('utf8')

TEMPLATE = '''
<!DOCTYPE html>
<html>
<head>
    <meta charset="UTF-8">
    <title>Nanopore测序数据统计分析报告</title>
</head>
<body>
    {body}
</body>
</html>
'''

def buildDOC(body):
    """
    :param body: body of TEMPLATE
    """
    try:
        html = TEMPLATE.format(
            body=body
        )  # 向模板中填充数据
        return html
    except Exception as err:
        raise err


def write_to_file(html, file):
    """
    write html to file
    """
    with open(file, 'w') as f:
        f.write(html)


def statistics(outdir, prefix=None):
    stat_raw = defaultdict(lambda: defaultdict(lambda: []))
    stat_filtered = defaultdict(lambda: defaultdict(lambda: []))
    len_dist = []

    if os.path.isdir(os.path.join(outdir, 'quality_control')):
        for item in os.listdir(os.path.join(outdir, 'quality_control')):
            if item.endswith('_QCstat.out') and not item.startswith("filtered_"):
                prefix = item.replace('_QCstat.out', '')
                if prefix == 'none':
                    continue
                with open(os.path.join(outdir, 'quality_control', item)) as f:
                    for line in f:
                        if line.startswith('# Fastq stats for'):
                            read_len = int((line.strip().split('reads >= '))[-1].replace('bp', ''))
                            len_dist.append(read_len)
                            for i in range(10):
                                line = next(f)
                                tmp = line.strip().split(': ')[-1]
                                if line.startswith('%') or line.startswith('meanLen'):
                                    tmp = round(float(tmp), 2)
                                stat_raw[prefix][read_len].append(tmp)
            elif item.endswith('_QCstat.out') and item.startswith("filtered_"):
                prefix = item.replace('_QCstat.out', '').replace('filtered_', '')
                if prefix == 'none':
                    continue
                with open(os.path.join(outdir, 'quality_control', item)) as f:
                    for line in f:
                        if line.startswith('# Fastq stats for'):
                            read_len = int((line.strip().split('reads >= '))[-1].replace('bp', ''))
                            len_dist.append(read_len)
                            for i in range(10):
                                line = next(f)
                                tmp = line.strip().split(': ')[-1]
                                if line.startswith('%') or line.startswith('meanLen'):
                                    tmp = round(float(tmp), 2)
                                stat_filtered[prefix][read_len].append(tmp)
    elif os.path.exists(os.path.join(outdir, 'QCstat.out')) and os.path.exists(os.path.join(outdir, 'filtered_QCstat.out')):
        if prefix is None:
            prefix = 'SAMPLE'

        with open(os.path.join(outdir, 'QCstat.out')) as f:
            for line in f:
                if line.startswith('# Fastq stats for'):
                    read_len = int((line.strip().split('reads >= '))[-1].replace('bp', ''))
                    len_dist.append(read_len)
                    for i in range(10):
                        line = next(f)
                        tmp = line.strip().split(': ')[-1]
                        if line.startswith('%') or line.startswith('meanLen'):
                            tmp = round(float(tmp), 2)
                        stat_raw[prefix][read_len].append(tmp)

        with open(os.path.join(outdir, 'filtered_QCstat.out')) as f:
            for line in f:
                if line.startswith('# Fastq stats for'):
                    read_len = int((line.strip().split('reads >= '))[-1].replace('bp', ''))
                    len_dist.append(read_len)
                    for i in range(10):
                        line = next(f)
                        tmp = line.strip().split(': ')[-1]
                        if line.startswith('%') or line.startswith('meanLen'):
                            tmp = round(float(tmp), 2)
                        stat_filtered[prefix][read_len].append(tmp)
    else:
        print('Not found "quality_control" dir or stat file: "QCstat.out" and "filtered_QCstat.out" in dir {}.'.format(outdir))
        sys.exit(-1)

    with open(os.path.join(outdir, 'raw_reads_stat.xls'), 'w') as f:

        f.write('sample\treads >=\tnumReads\t%totalNumReads\tnumBasepairs\t%totalBasepairs\tmeanLen\tmedianLen\tminLen\tmaxLen\tN50\tL50\n')

        for key in sorted(stat_raw.keys()):
            for subkey in sorted(set(len_dist)):
                if subkey in stat_raw[key].keys():
                    f.write('{}\t>={}kb\t{}\n'.format(key, round(subkey/1000, 2), '\t'.join(map(str, stat_raw[key][subkey]))))
                else:
                    f.write('{}\t>={}kb\t{}\n'.format(key, round(subkey/1000, 2), '\t'.join(['0'] * 10)))

    with open(os.path.join(outdir, 'filtered_reads_stat.xls'), 'w') as f:
        f.write('sample\treads >=\tnumReads\t%totalNumReads\tnumBasepairs\t%totalBasepairs\tmeanLen\tmedianLen\tminLen\tmaxLen\tN50\tL50\n')
        for key in sorted(stat_filtered.keys()):
            for subkey in sorted(set(len_dist)):
                if subkey in stat_filtered[key].keys():
                    f.write('{}\t>={}kb\t{}\n'.format(key, round(subkey/1000, 2), '\t'.join(map(str, stat_filtered[key][subkey]))))
                else:
                    f.write('{}\t>={}kb\t{}\n'.format(key, round(subkey/1000, 2), '\t'.join(['0'] * 10)))


def read_table(file):
    """
    convert to html format table
    """
    data = {}
    # df = pd.DataFrame(data)
    with open(file, 'r') as f:
        header = f.readline().replace('\n', '').split('\t')
        for item in header:
            data[item] = []
        for line in f.readlines():
            line = line.replace('\n', '')
            temp = line.split('\t')
            for i in range(len(temp)):
                data[header[i]].append(temp[i])
    return data, header


def to_html(raw_stat, filtered_stat, min_quality=7, min_len=2000, max_len=None, png=None):
    localtime = time.strftime("%Y-%m-%d", time.localtime())
    body = '<div style="text-align: center;vertical-align: middle;">'
    if max_len is not None:
        start = '2）过滤低质量read：(a) 剔除平均质量值Q小于{}的read；(b) 剔除长度小于{} bp或长度大于{} bp的read。'.format(min_quality, min_len, max_len)
    else:
        start = '2）过滤低质量read：(a) 剔除平均质量值Q小于{}的read；(b) 剔除长度小于{} bp的read。'.format(min_quality, min_len)

    body += ('<img src="./logo.jpg" height="8%" width="8%" style="float:left;"><h4 style="float:right;">{}</h4>\n'
        '<h1 align="center">Nanopore测序数据基本统计分析报告</h1>\n'
        '<hr style="FILTER: alpha(opacity=100,finishopacity=0,style=2)" width="100%" color=#A2CD5A SIZE=10>\n'
        '<h2>===== 质控分析简介 ====</h2>\n'
        '<hr style="border:3px dashed #A2CD5A; height:3px" SIZE=3 width="80%" >\n'
        '<hr style="border:1px dashed #A2CD5A" width="80%" color=#A2CD5A SIZE=3>\n'
        '<hr style="border:1px dashed #A2CD5A" width="80%" color=#A2CD5A SIZE=3>\n'
        '<p align="left" style="font-size:120%">\n'
        '<br />{}本流程针对Nanopore原始下机数据进行处理，支持fast5或fastq两种格式。基本步骤包括：'
        '<br />{}1）去接头，拆分barcode标签（可选）：利用Qcat软件去除接头序列，同时利用Guppy对不同样本进行拆分，得到各样本的测序数据(如果测序过程中添加了barcode标签)。\n'
        '<br />{}{}\n'
        '<br />{}3）统计基本信息：包括原始测序数据和过滤后的测序数据两部分，具体统计信息如下。</p>\n'
        '<h2>===== 原始测序数据 ====</h2>\n'
        '<hr style="border:3px dashed #A2CD5A; height:3px" SIZE=3 width="80%" >\n'
        '<hr style="border:1px dashed #A2CD5A" width="80%" color=#A2CD5A SIZE=3>\n'
        '<hr style="border:1px dashed #A2CD5A" width="80%" color=#A2CD5A SIZE=3>\n'
        '<h3>1. 基本信息统计表</h3>\n'
        '<hr style="border:3px dashed #A2CD5A; height:3px" SIZE=3 width="50%">\n'
        '<hr style="border:1px dashed #A2CD5A" width="50%" color=#A2CD5A SIZE=3>\n'
        ).format(localtime, '&nbsp' * 30, '&nbsp' * 30, '&nbsp' * 30, '&nbsp' * 30, start, '&nbsp' * 30)
    raw_data, raw_header = read_table(raw_stat)
    raw_df = pandas.DataFrame(raw_data, columns=raw_header)
    # body += raw_df.to_html(index=False)
    tmp = raw_df.to_html(index=False)
    tmp = re.sub('<table border="1" class="dataframe">', '<table border="1" width="80%" class="dataframe" style="text-align: right; margin:auto"><caption>表1：原始测序数据基本信息统计表</caption>', tmp)
    body += tmp

    body += ('<h3>2. Reads长度-质量值分布图</h3>\n'
            '<hr style="border:3px dashed #A2CD5A; height:3px" SIZE=3 width="50%">\n'
            '<hr style="border:1px dashed #A2CD5A" width="50%" color=#A2CD5A SIZE=3>\n')
    # body = '<style> img {width: 100px;} </style>\n'

    if os.path.isdir(os.path.join(os.path.dirname(raw_stat), 'quality_control')):
        qc_path = os.path.join(os.path.dirname(raw_stat), 'quality_control')
        for item in os.listdir(qc_path):
            if item.endswith('.png') and not item.startswith('filtered_'):
                if item.startswith('none_') or item.startswith('filtered_none_'):
                    continue
                # body += '<img src=\"{}\">\n'.format(os.path.join(qc_path, item))
                body += '<img src=\"./{}\" height="40%" width="40%"><br />\n'.format(os.path.join('quality_control', item))
    else:
        qc_path = os.path.dirname(raw_stat)
        for item in os.listdir(qc_path):
            if item.endswith('.png') and not item.startswith('filtered_'):
                if item.startswith('none_') or item.startswith('filtered_none_'):
                    continue
                # body += '<img src=\"{}\">\n'.format(os.path.join(qc_path, item))
                body += '<img src=\"./{}\" height="40%" width="40%">\n'.format(item)


    body += (
        '<hr style="border:1px dashed #A2CD5A" width="80%" color=#A2CD5A SIZE=3>\n'
        '<hr style="border:1px dashed #A2CD5A" width="80%" color=#A2CD5A SIZE=3>\n'
        '<hr style="border:3px dashed #A2CD5A; height:3px" SIZE=3 width="80%" >\n'
        '<h2>===== 过滤后测序数据 ====</h2>\n'
        '<hr style="border:3px dashed #A2CD5A; height:3px" SIZE=3 width="80%" >\n'
        '<hr style="border:1px dashed #A2CD5A" width="80%" color=#A2CD5A SIZE=3>\n'
        '<hr style="border:1px dashed #A2CD5A" width="80%" color=#A2CD5A SIZE=3>\n'
        '<h3>1. 基本信息统计表</h3>\n'
        '<hr style="border:3px dashed #A2CD5A; height:3px" SIZE=3 width="50%">\n'
        '<hr style="border:1px dashed #A2CD5A" width="50%" color=#A2CD5A SIZE=3>\n')
    filtered_data, filtered_header = read_table(filtered_stat)
    filtered_df = pandas.DataFrame(filtered_data, columns=filtered_header)
    # body += filtered_df.to_html(index=False)
    tmp = filtered_df.to_html(index=False)
    tmp = re.sub('<table border="1" class="dataframe">', '<table border="1" width="80%" class="dataframe" style="text-align: right; margin:auto"><caption>表2：过滤后测序数据基本信息统计表</caption>', tmp)
    body += tmp

    body += ('<h3>2. Reads长度-质量值分布图</h3>\n'
            '<hr style="border:3px dashed #A2CD5A; height:3px" SIZE=3 width="50%">\n'
            '<hr style="border:1px dashed #A2CD5A" width="50%" color=#A2CD5A SIZE=3>\n')

    if os.path.isdir(os.path.join(os.path.dirname(raw_stat), 'quality_control')):
        qc_path = os.path.join(os.path.dirname(raw_stat), 'quality_control')
        for item in os.listdir(qc_path):
            if item.endswith('.png') and item.startswith('filtered_'):
                if item.startswith('none_') or item.startswith('filtered_none_'):
                    continue
                # body += '<img src=\"{}\">\n'.format(os.path.join(qc_path, item))
                body += '<img src=\"./{}\" height="40%" width="40%"><br />\n'.format(os.path.join('quality_control', item))
    else:
        qc_path = os.path.dirname(raw_stat)
        for item in os.listdir(qc_path):
            if item.endswith('.png') and item.startswith('filtered_'):
                if item.startswith('none_') or item.startswith('filtered_none_'):
                    continue
                # body += '<img src=\"{}\">\n'.format(os.path.join(qc_path, item))
                body += '<img src=\"./{}\" height="40%" width="40%">\n'.format(item)

    body += '</div>'
    html_out = buildDOC(body)
    write_to_file(html_out, os.path.join(os.path.dirname(raw_stat), 'reads_stat.html'))


def main(outdir, min_quality=7, min_len=2000, max_len=None):
    statistics(outdir)
    to_html(os.path.join(outdir, 'raw_reads_stat.xls'), os.path.join(outdir, 'filtered_reads_stat.xls'), min_quality, min_len, max_len)


if __name__ == '__main__':
    main(sys.argv[1])



from __future__ import print_function
import sys


def pre_list(strain_list_file):
	strain_info = dict()
	with open(strain_list_file, 'r') as f:
		for line in f.readlines():
			line = line.replace('\n', '')
			tmp = line.split('\t')
			columns = len(tmp)

			if (columns < 2) or (columns > 2 and columns < 4):
				print('Error: 2 or 4 tab-delimited columns : barcodes strain_name [reads1 path] [reads2 path]')
				sys.exit(-1)

			if tmp[0] not in strain_info.keys():
				strain_info[tmp[0]] = []

			strain_info[tmp[0]].extend(tmp)

	return strain_info, columns




#!/usr/bin/python
# -*- coding: utf-8 -*-

import argparse
import os
import re
import sys

from version import __version__
import basic_data_analysis
import assembly
import pre
import report

def get_arguments():
    """
    Parse the command line arguments.
    """
    parser = argparse.ArgumentParser(usage='Use "python %(prog)s -h/--help" for more information')
    parser.add_argument('-i', '--input', help='Fastq file path or Folder path containing fast5 or fastq files', required=True)
    parser.add_argument('-s', '--strain_list', help='2 columns or 4 columns, a tab-delimited list containing barcodes, strain names and/or PE reads1 path and reads2 path', required=True)
    parser.add_argument('-b', '--barcoding', help='Search for barcodes to demultiplex sequencing data', action='store_true')
    # parser.add_argument('-1', '--short_read1', help='FASTQ file of first short reads in each pair (optional)')
    # parser.add_argument('-2', '--short_read2', help='FASTQ file of second short reads in each pair (optional)')
    parser.add_argument('-g', '--genomesize', help='genome size, <number>[g|m|k], for example: 4800000, 48000k, 4.8m', default=None)
    # parser.add_argument('-p', '--prefix', default='sample', help='Prefix name (default: sample)')
    # parser.add_argument('--format', help='Input file format, fast5 or fastq (default: fastq)', choices=('fast5', 'fastq'), default='fastq')
    parser.add_argument('-t', '--thread', help='use NUM threads (default: 16)', type=int, default=16)
    parser.add_argument('-o', '--outdir', help='Output dir (default: None)', default=None)
    parser.add_argument('--step', help='Analysis steps: only basic data analysis [1], only assembly [2], or basic data analysis and assembly [all]', choices=('1', '2', 'all'), default='all')
    parser.add_argument('-f', '--flowcell', help='Flowcell used during the sequencing run (default: FLO-MIN106)', default='FLO-MIN106')
    parser.add_argument('-k', '--kit', help='Kit used during the sequencing run (default: SQK-LSK109)', default='SQK-LSK109')
    parser.add_argument('--barcode_kit', help='Barcode Kits used during the sequencing run (default: EXP-NBD104)', default='EXP-NBD104')
    parser.add_argument('-q', '--min_quality', help='Filter on a minimum average read quality score (default: 7)', default=7, type=int)
    parser.add_argument('-l', '--min_length', help='Filter on a minimum read length (default: 2000)', default=2000, type=int)
    parser.add_argument('-m', '--max_length', help='Filter on a maximum read length (default: 100000000)', default=100000000, type=int)
    # parser.add_argument('--minreadlength', help='Filter on a maximum read length (default: 100000000)', default=100000000, type=int)
    parser.add_argument('--minoverlaplength', help='Canu assembly: ignore read-to-read overlaps shorter than "number" bases long (default: 500)', default=500, type=int)
    parser.add_argument('-v', '--version', action='version', version=__version__, help="Show program's version number and exit")
    args = parser.parse_args()

    return args


def main():
    args = get_arguments()

    strain_info, columns = pre.pre_list(args.strain_list)

    if columns == 2:
        if args.genomesize is None:
            print('Error: parameter "genomesize" is required when using canu software for long-read-only assembly.\n')
            sys.exit(-1)


    if args.step == '1':
       # Step1: Basci_Data_Analysis
        basic_data_analysis_outdir = os.path.join(args.outdir, '01.Basic_Data_Analysis')

        basic_data_analysis.basic_data_analysis(input=args.input, strain_barcode_dict=strain_info, outdir=basic_data_analysis_outdir, 
            flowcell=args.flowcell, kit=args.kit, barcode_kits=args.barcode_kit,
            thread=args.thread, barcoding=args.barcoding, min_quality=args.min_quality, 
            min_length=args.min_length, max_length=args.max_length)

        os.system('cp {} {}'.format(os.path.join(os.path.dirname(os.path.abspath(sys.argv[0])), 'logo.jpg'), basic_data_analysis_outdir))
        report.main(basic_data_analysis_outdir, min_quality=args.min_quality, min_len=args.min_length, max_len=args.max_length)
    elif args.step == '2':
        # Setp1: Assembly (Skip Basic Data Analysis)

        assembly_outdir = os.path.join(args.outdir, '02.Assembly')

        if not os.path.isdir(assembly_outdir):
            os.makedirs(assembly_outdir)

        assembly.assembly_one(args.input, strain_info=strain_info, thread=args.thread, genomesize=args.genomesize, outdir=assembly_outdir, minReadLength=args.min_length, minOverlapLength=args.minoverlaplength)
        # Step3:     
        assembly.annotation(strain_info, args.outdir)
    else:
        # Step1: Basci_Data_Analysis
        basic_data_analysis_outdir = os.path.join(args.outdir, '01.Basic_Data_Analysis')

        basic_data_analysis.basic_data_analysis(input=args.input, strain_barcode_dict=strain_info, outdir=basic_data_analysis_outdir, 
            flowcell=args.flowcell, kit=args.kit, barcode_kits=args.barcode_kit,
            thread=args.thread, barcoding=args.barcoding, min_quality=args.min_quality, 
            min_length=args.min_length, max_length=args.max_length)

        os.system('cp {} {}'.format(os.path.join(os.path.dirname(os.path.abspath(sys.argv[0])), 'logo.jpg'), basic_data_analysis_outdir))

        report.main(basic_data_analysis_outdir, min_quality=args.min_quality, min_len=args.min_length, max_len=args.max_length)

        # Step2: Assembly
        assembly_outdir = os.path.join(args.outdir, '02.Assembly')

        if not os.path.isdir(assembly_outdir):
            os.makedirs(assembly_outdir)

        if args.barcoding:
            nanopore_fq_file = os.path.join(basic_data_analysis_outdir, 'barcode_demultiplexing_data')
        else:
            nanopore_fq_file = basic_data_analysis_outdir
        assembly.assembly_all(nanopore_fq_file, strain_info=strain_info, thread=args.thread, genomesize=args.genomesize, outdir=assembly_outdir, minReadLength=args.min_length, minOverlapLength=args.minoverlaplength)

        # Step3:     
        assembly.annotation(strain_info, args.outdir)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding:utf-8 -*-
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

version = '0.1.0'

setup(
    name='zctestpy',
    version=version,
    description='X-ABT Nanopore Assembly Pipeline',
    # long_description=readme,
    packages=['zcpy'],
    # install_requires=[
    #     'PyYAML==3.11',
    #     'requests==2.5.3'
    # ],
    entry_points={
        'console_scripts': [
            'zctestpy = zcpy.main:main',
        ]
    },
    include_package_data=True,
    classifiers=[
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
    ]
)